using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using Foo.DataAccessLayer;
using Foo.DataAccessLayer.DataTypes;

namespace Foo.WorkspaceMgr
{
    public interface IWorkspaceMgr
    {
        // Artifact Related
        bool LaunchArtifact(ArtifactItem artifactItem);
        bool CloseArtifact(ArtifactItem artifactItem);
        bool HideShowArtifact(ArtifactItem artifactItem, bool bShow);

        // Workspace Related
        bool LaunchWorkspace(string WorkspaceName);
        bool CloseWorkspace(string WorkspaceName);
        bool HideShowWorkspace(string WorkspaceName, bool bShow);
    }
}
